"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.useModal = void 0;
var jsx_runtime_1 = require("react/jsx-runtime");
var react_dom_1 = __importDefault(require("react-dom"));
var events_1 = require("./events");
var helpers_1 = require("./helpers");
function useModal(_a) {
    var Component = _a.Component, overlayStyles = _a.overlayStyles, overlayClassName = _a.overlayClassName, closeOnEsc = _a.closeOnEsc, closeOnOverlayClick = _a.closeOnOverlayClick, defaultResolved = _a.defaultResolved, overlayClassNameOnClose = _a.overlayClassNameOnClose, overlayClassNameOnOpen = _a.overlayClassNameOnOpen, closeTimeoutMs = _a.closeTimeoutMs, blockBodyScroll = _a.blockBodyScroll, onOpen = _a.onOpen, onClose = _a.onClose;
    var addPortal = events_1.useAddPortal();
    var removePortal = events_1.useRemovePortal();
    var onResolve = function (resolve, containerIdPostfix) {
        return function (result) {
            var beforeCloseActions = function () {
                onClose && onClose({ resolved: result });
                var modalContainer = helpers_1.getModalContainerById(containerIdPostfix);
                helpers_1.removeClassesFromModalContainer(modalContainer, overlayClassNameOnOpen);
                helpers_1.addClassesToModalContainer(modalContainer, overlayClassNameOnClose);
            };
            var afterCloseActions = function () {
                cleanupContainer(containerIdPostfix);
                resolve(result);
            };
            beforeCloseActions();
            closeTimeoutMs ? setTimeout(afterCloseActions, closeTimeoutMs) : afterCloseActions();
        };
    };
    var showModal = function () {
        return new Promise(function (resolve) {
            createModal(resolve);
        });
    };
    var addEscListener = function (containerIdPostfix, onResolve) {
        var container = helpers_1.getModalContainerById(containerIdPostfix);
        // TODO: Find method to not shown error while ResultType is void
        // if (!defaultResolved) {
        //   console.warn(
        //     "use-async-modal: when options `closeOnEsc` is true, `defaultResolved` must be set!",
        //   );
        // }
        if (!container) {
            return;
        }
        container.addEventListener("keydown", function _onEsc(e) {
            if (e.key === "Escape") {
                onResolve(defaultResolved);
                container.removeEventListener("keydown", _onEsc);
            }
        });
    };
    var addClickListener = function (containerIdPostfix, onResolve) {
        var container = helpers_1.getModalContainerById(containerIdPostfix);
        // TODO: Find method to not shown error while ResultType is void
        // if (!defaultResolved) {
        //   console.warn(
        //     "use-async-modal: when options `closeOnOverlayClick` is true, `defaultResolved` must be set!",
        //   );
        // }
        if (!container) {
            return;
        }
        container.addEventListener("click", function _onClick(e) {
            if (e.target === container) {
                onResolve(defaultResolved);
                container.removeEventListener("click", _onClick);
            }
        });
    };
    var createModal = function (resolve) {
        var containerIdPostfix = helpers_1.getRandomPostfix();
        var body = document.querySelector("body");
        var modalContainer = document.createElement("div");
        modalContainer.setAttribute("id", "modal__" + containerIdPostfix);
        modalContainer.setAttribute("role", "dialog");
        modalContainer.setAttribute("aria-modal", "true");
        modalContainer.tabIndex = -1;
        // Apply styles
        modalContainer.classList.add("useModal__overlay");
        overlayStyles && modalContainer.setAttribute("style", helpers_1.cssPropertiesToString(overlayStyles));
        helpers_1.addClassesToModalContainer(modalContainer, overlayClassName);
        helpers_1.addClassesToModalContainer(modalContainer, overlayClassNameOnOpen);
        body === null || body === void 0 ? void 0 : body.appendChild(modalContainer);
        var onOpenOptions = {
            containerRef: modalContainer,
            containerId: containerIdPostfix,
            blockBodyScroll: blockBodyScroll !== null && blockBodyScroll !== void 0 ? blockBodyScroll : true,
        };
        var handleResolve = onResolve(resolve, containerIdPostfix);
        var containerRef = __assign(__assign({}, onOpenOptions), { portal: react_dom_1.default.createPortal(jsx_runtime_1.jsx(Component, { onResolve: handleResolve }, void 0), modalContainer, containerIdPostfix) });
        addPortal(containerRef);
        closeOnEsc && addEscListener(onOpenOptions.containerId, handleResolve);
        closeOnOverlayClick && addClickListener(onOpenOptions.containerId, handleResolve);
        onOpen && onOpen(onOpenOptions);
    };
    var cleanupContainer = function (containerIdPostfix) {
        var body = document.querySelector("body");
        var modalContainer = helpers_1.getModalContainerById(containerIdPostfix);
        removePortal(containerIdPostfix);
        if (!modalContainer || !body) {
            return;
        }
        body.removeChild(modalContainer);
    };
    return showModal;
}
exports.useModal = useModal;
